@@ -5,7 +5,7 @@ class DeflectionModel(ABC):
     args4deflection = ["ct_ilk"]
 
     @abstractmethod
-    def calc_deflection(self, dw_ijl, hcw_ijl, **kwargs):
+    def calc_deflection(self, dw_ijl, hcw_ijl, dh_ijl, **kwargs):
         """Calculate deflection
 
         This method must be overridden by subclass
@@ -23,4 +23,7 @@ class DeflectionModel(ABC):
         hcw_ijlk : array_like
             horizontal crosswind distance from source wind turbine(i) to destination wind turbine/site (j)
             for all wind direction (l) and wind speed (k)
+        dh_ijlk : array_like
+            vertical distance from source wind turbine(i) to destination wind turbine/site (j)
+            for all wind direction (l) and wind speed (k)
         """
