@@ -1,7 +1,10 @@
-import numpy as np
-from abc import ABC, abstractmethod
-import py_wake.utils.xarray_utils  # register ilk function @UnusedImport
 import matplotlib.pyplot as plt
+import numpy as np
+from py_wake.site.shear import PowerShear
+import py_wake.utils.xarray_utils  # register ilk function @UnusedImport
+import xarray as xr
+from abc import ABC, abstractmethod
+from py_wake.utils.xarray_utils import da2py
 
 """
 suffixs:
@@ -11,15 +14,12 @@ suffixs:
 - k: Wind speeds
 - m: Height above ground
 """
-from py_wake.site.distance import StraightDistance
-from py_wake.site.shear import PowerShear
-import xarray as xr
 
 
 class LocalWind(xr.Dataset):
     __slots__ = ('wd_bin_size')
 
-    def __init__(self, x_i, y_i, h_i, wd, ws, wd_bin_size, WD=None, WS=None, TI=None, P=None):
+    def __init__(self, x_i, y_i, h_i, wd, ws, time, wd_bin_size, WD=None, WS=None, TI=None, P=None):
         """
 
         Parameters
@@ -33,7 +33,15 @@ class LocalWind(xr.Dataset):
         P : array_like
             Probability/weight
         """
-        coords = {'wd': wd, 'ws': ws, }
+        ws = np.atleast_1d(ws)
+        if time is not False:
+            assert len(wd) == len(ws)
+            if time is True:
+                time = np.arange(len(wd))
+            coords = {'time': time, 'wd': ('time', wd), 'ws': ('time', ws)}
+        else:
+            coords = {'wd': wd, 'ws': np.atleast_1d(ws)}
+
         assert len(np.atleast_1d(x_i)) == len(np.atleast_1d(y_i))
         n_i = max(len(np.atleast_1d(x_i)), len(np.atleast_1d(h_i)))
         coords['i'] = np.arange(n_i)
@@ -41,10 +49,9 @@ class LocalWind(xr.Dataset):
         for k, v in [('x', x_i), ('y', y_i), ('h', h_i)]:
             if v is not None:
                 coords[k] = ('i', np.zeros(n_i) + v)
-
-        xr.Dataset.__init__(self, data_vars={k: v for k, v in [('WD', WD), ('WS', WS),
-                                                               ('TI', TI), ('P', P)] if v is not None},
-                            coords=coords)
+        xr.Dataset.__init__(self, data_vars={k: da2py(v, include_dims=True) for k, v in [('WD', WD), ('WS', WS),
+                                                                                         ('TI', TI), ('P', P)] if v is not None},
+                            coords={k: da2py(v) for k, v in coords.items()})
         self.attrs['wd_bin_size'] = wd_bin_size
 
         # set localWind.WS_ilk etc.
@@ -90,6 +97,15 @@ class Site(ABC):
         self.default_ws = np.arange(3, 26.)
         self.default_wd = np.arange(360)
 
+    @property
+    def distance(self):
+        return self._distance
+
+    @distance.setter
+    def distance(self, distance):
+        self._distance = distance
+        distance.site = self
+
     def get_defaults(self, wd=None, ws=None):
         if wd is None:
             wd = self.default_wd
@@ -101,15 +117,7 @@ class Site(ABC):
             ws = np.atleast_1d(ws)
         return wd, ws
 
-    def wref(self, wd, ws, ws_bins=None):
-        wd, ws = self.get_defaults(wd, ws)
-        WS = xr.DataArray(ws, [('ws', ws)])
-        ds = self.ws_bins(WS, ws_bins)
-        ds['WS'] = WS
-        ds['WD'] = xr.DataArray(wd, [('wd', wd)])
-        return ds
-
-    def local_wind(self, x_i, y_i, h_i=None, wd=None, ws=None, wd_bin_size=None, ws_bins=None):
+    def local_wind(self, x_i, y_i, h_i=None, wd=None, ws=None, time=False, wd_bin_size=None, ws_bins=None):
         """Local free flow wind conditions
 
         Parameters
@@ -124,6 +132,9 @@ class Site(ABC):
             Global wind direction(s). Override self.default_wd
         ws : float, int or array_like, optional
             Global wind speed(s). Override self.default_ws
+        time : boolean or array_like
+            If True or array_like, wd and ws is interpreted as a time series
+            If False, full wd x ws matrix is computed
         wd_bin_size : int or float, optional
             Size of wind direction bins. default is size between first and
             second element in default_wd
@@ -144,7 +155,7 @@ class Site(ABC):
         """
         wd, ws = self.get_defaults(wd, ws)
         wd_bin_size = self.wd_bin_size(wd, wd_bin_size)
-        lw = LocalWind(x_i, y_i, h_i, wd, ws, wd_bin_size)
+        lw = LocalWind(x_i, y_i, h_i, wd, ws, time, wd_bin_size)
         return self._local_wind(lw, ws_bins)
 
     @abstractmethod
@@ -171,57 +182,8 @@ class Site(ABC):
                 Probability/weight
         """
 
-    @abstractmethod
-    def probability(self, localWind):
-        """Probability of wind situation (wind speed and direction)
-
-        Parameters
-        ----------
-        localWind : LocalWind
-
-        Returns
-        -------
-        P : DataArray
-            Probability of wind speed and direction at local positions
-        """
-
-    def distances(self, src_x_i, src_y_i, src_h_i, dst_x_j, dst_y_j, dst_h_j, wd_il):
-        """Calculate down/crosswind distance between source and destination points
-
-        Parameters
-        ----------
-        src_x_i : array_like
-            Source x position
-        src_y_i : array_like
-            Source y position
-        src_h_i : array_like
-            Source height above ground level
-        dst_x_j : array_like
-            Destination x position
-        dst_y_j : array_like
-            Destination y position
-        dst_h_j : array_like
-            Destination height above ground level
-        wd_il : array_like, shape (#src, #wd)
-            Local wind direction at the source points for all global wind directions
-
-
-        Returns
-        -------
-        dw_ijl : array_like
-            down wind distances. Positive downstream
-        hcw_ijl : array_like
-            horizontal cross wind distances. Positive when the wind is in your
-            back and  the turbine lies on the left.
-        dh_ijl : array_like
-            vertical distances
-        dw_order_indices_l : array_like
-            indices that gives the downwind order of source points
-        """
-        return self.distance(self, src_x_i, src_y_i, src_h_i, dst_x_j, dst_y_j, dst_h_j, wd_il)
-
-    def wt2wt_distances(self, x_i, y_i, h_i, wd_il):
-        return self.distances(x_i, y_i, h_i, x_i, y_i, h_i, wd_il)
+    def wt2wt_distances(self, WD_il):
+        return self.distance(WD_il)
 
     @abstractmethod
     def elevation(self, x_i, y_i):
@@ -286,7 +248,8 @@ class Site(ABC):
             sector[lo + 1:up + 1] = i
         return sector
 
-    def plot_ws_distribution(self, x=0, y=0, h=70, wd=[0], include_wd_distribution=False, ax=None):
+    def plot_ws_distribution(self, x=0, y=0, h=70, wd=[0], ws=np.arange(0.05, 30.05, .1),
+                             include_wd_distribution=False, ax=None):
         """Plot wind speed distribution
 
         Parameters
@@ -299,6 +262,8 @@ class Site(ABC):
             Local height above ground
         wd : int or array_like
             Wind direction(s) (one curve pr wind direction)
+        ws : array_like, optional
+            Wind speeds to calculate for
         include_wd_distributeion : bool, default is False
             If true, the wind speed probability distributions are multiplied by
             the wind direction probability. The sector size is set to 360 / len(wd).
@@ -307,9 +272,8 @@ class Site(ABC):
 
         """
         if ax is None:
-
             ax = plt
-        ws = np.arange(0.05, 30.05, .1)
+
         lbl = "Wind direction: %d deg"
         if include_wd_distribution:
 
@@ -372,8 +336,12 @@ class Site(ABC):
         ax.set_theta_offset(np.pi / 2.0)
 
         if ws_bins is None:
-            WS = xr.DataArray([100], [('ws', [100])])
-            lw = self.local_wind(x_i=x, y_i=y, h_i=h, wd=np.arange(360), ws=[100], ws_bins=[0, 200], wd_bin_size=1)
+            if any(['ws' in v.dims for v in self.ds.data_vars.values()]):
+                lw = self.local_wind(x_i=x, y_i=y, h_i=h, wd=np.arange(360), wd_bin_size=1)
+                lw['P'] = lw.P.sum('ws')
+            else:
+                lw = self.local_wind(x_i=x, y_i=y, h_i=h, wd=np.arange(360),
+                                     ws=[100], ws_bins=[0, 200], wd_bin_size=1)
         else:
             if not hasattr(ws_bins, '__len__'):
                 ws_bins = np.linspace(0, 30, ws_bins)
@@ -404,99 +372,9 @@ class Site(ABC):
         return p.T
 
 
-class UniformSite(Site):
-    """Site with uniform (same wind over all, i.e. flat uniform terrain) and
-    constant wind speed probability of 1. Only for one fixed wind speed
-    """
-
-    def __init__(self, p_wd, ti, ws=12, interp_method='nearest', shear=None):
-        super().__init__(StraightDistance())
-        self.default_ws = ws
-        self.ti = get_sector_xr(ti, 'Turbulence intensity')
-        self.p_wd = get_sector_xr(p_wd / np.mean(p_wd) / 360, "Probability of wind direction +/- 0.5deg")
-        self.interp_method = interp_method.replace('piecewise', 'nearest').replace('spline', 'cubic')
-        if self.interp_method not in ['linear', 'nearest', 'zero', 'slinear', 'quadratic', 'cubic', 'previous', 'next']:
-            raise NotImplementedError('interp_method=%s not implemented' % interp_method)
-        self.shear = shear
-
-    def probability(self, localWind):
-        P = self.p_wd.interp(wd=localWind.WD, method=self.interp_method) * localWind.wd_bin_size
-        P.attrs['Description'] = "Probability of wind flow case (i.e. wind direction +/-%fdeg)" % (
-            localWind.wd_bin_size / 2)
-        return P
-
-    def _local_wind(self, localWind, ws_bins=None):
-        lw = localWind
-        if self.shear:
-            assert 'h' in lw and np.all(lw.h.data != None), "Height must be specified and not None"  # nopep8
-            h = np.unique(lw.h)
-            if len(h) > 1:
-                h = lw.h
-            WS = self.shear(lw.ws, lw.wd, h)
-        else:
-            WS = lw.ws
-
-        TI = self.ti.interp_all(lw)
-        lw.set_W(WS, lw.wd, TI, ws_bins)
-
-        lw['P'] = self.probability(lw)
-        return lw
-
-    def elevation(self, x_i, y_i):
-        return np.zeros_like(x_i)
-
-
-class UniformWeibullSite(UniformSite):
-    """Site with uniform (same wind over all, i.e. flat uniform terrain) and
-    weibull distributed wind speed
-    """
-
-    def __init__(self, p_wd, a, k, ti, interp_method='nearest', shear=None):
-        """Initialize UniformWeibullSite
-
-        Parameters
-        ----------
-        p_wd : array_like
-            Probability of wind direction sectors
-        a : array_like
-            Weilbull scaling parameter of wind direction sectors
-        k : array_like
-            Weibull shape parameter
-        ti : float or array_like
-            Turbulence intensity
-        interp_method : 'nearest', 'linear' or 'spline'
-            p_wd, a, k, ti and alpha are interpolated to 1 deg sectors using this
-            method
-        shear : Shear object
-            Shear object, e.g. NoShear(), PowerShear(h_ref, alpha)
-
-        Notes
-        ------
-        The wind direction sectors will be: [0 +/- w/2, w +/- w/2, ...]
-        where w is 360 / len(p_wd)
-
-        """
-        super().__init__(p_wd, ti, interp_method=interp_method, shear=shear)
-        self.default_ws = np.arange(3, 26)
-        self.a = get_sector_xr(a, 'Weibull A')
-        self.k = get_sector_xr(k, 'Weibull k')
-
-    def weibull_weight(self, localWind, A, k):
-        def cdf(ws, A=A, k=k):
-            return 1 - np.exp(-(1 / A * ws) ** k)
-
-        P = cdf(localWind.ws_upper) - cdf(localWind.ws_lower)
-        P.attrs['Description'] = "Probability of wind flow case (i.e. wind direction and wind speed)"
-        return P
-
-    def probability(self, localWind):
-        lw = localWind
-        p_wd_ilk = UniformSite.probability(self, lw)
-
-        P_ilk = p_wd_ilk * self.weibull_weight(lw,
-                                               self.a.interp(wd=lw.WD, method=self.interp_method),
-                                               self.k.interp(wd=lw.WD, method=self.interp_method))
-        return P_ilk
+from py_wake.site import xrsite  # @NoMove # nopep8
+UniformSite = xrsite.UniformSite
+UniformWeibullSite = xrsite.UniformWeibullSite
 
 
 def get_sector_xr(v, name):
