@@ -21,9 +21,11 @@ except ModuleNotFoundError:
         try:
             self.nb = self.load_notebook(self.filename)
         except Exception as e:
-
             raise Exception('Error in ', os.path.relpath(filename)) from e
 
+    def __repr__(self):
+        return self.filename
+
     def load_notebook(self, filename):
         with open(filename, encoding='utf-8') as fid:
             nb = json.load(fid)
@@ -101,6 +103,13 @@ except ModuleNotFoundError:
             return line
 
         lines = [fix(l) for l in code.split("\n")]
+
+        # import * only allowed at module level
+        # So extract and remove from code lines
+        module_imports = [l for l in lines if l.startswith('from') and l.endswith('import *')]
+        for l in module_imports:
+            lines.remove(l)
+
         if len(lines) == 1 and lines[0] == '':
             return
         try:
@@ -108,10 +117,18 @@ except ModuleNotFoundError:
 
             with contextlib.redirect_stdout(StringIO()):
                 with contextlib.redirect_stderr(StringIO()):
-                    exec("def test():\n    " + "\n    ".join(lines) + "\ntest()", {}, {})
-                    plt.close()
+                    # execute module level imports (stored in l dict) and use a locals in execution of code
+                    g, l = {}, {}
+                    exec("\n".join(module_imports), g, l)
+
+                    code_str = "def test():\n    " + "\n    ".join(lines) + "\ntest()"
+                    exec(code_str, l, {})
         except Exception as e:
+            for i, l in enumerate(code_str.split("\n")):
+                print(i, l)
             raise type(e)("Code error in %s\n%s\n" % (self.filename, str(e))).with_traceback(sys.exc_info()[2])
+        finally:
+            plt.close('all')
 
     def check_links(self):
         txt = "\n".join(self.get_text())
