@@ -5,6 +5,7 @@ import pytest
 from py_wake.tests.notebook import Notebook
 import py_wake
 from py_wake.flow_map import Grid
+from pathlib import Path
 
 
 def get_notebooks():
@@ -14,9 +15,15 @@ def get_notebooks():
     return get(path) + get(path + "exercises/")
 
 
-@pytest.mark.parametrize("notebook", get_notebooks())
+notebooks = get_notebooks()
+
+
+@pytest.mark.parametrize("notebook", notebooks, ids=[os.path.basename(nb.filename) for nb in notebooks])
 def test_notebooks(notebook):
     import matplotlib.pyplot as plt
+    if (str(Path(notebook.filename).relative_to(os.path.dirname(py_wake.__file__) + "/../docs/notebooks/")) in
+            ['Gradients.ipynb']):
+        return
 
     def no_show(*args, **kwargs):
         pass
@@ -35,5 +42,9 @@ def test_notebooks(notebook):
         raise Exception(notebook.filename + " failed") from e
     finally:
         Grid.default_resolution = default_resolution
-        plt.close()
+        plt.close('all')
         plt.rcParams.update({'figure.max_open_warning': 20})
+
+
+if __name__ == '__main__':
+    print("\n".join([f.filename for f in get_notebooks()]))
