@@ -11,9 +11,20 @@ from py_wake.site.distance import TerrainFollowingDistance, StraightDistance, Te
 import math
 from py_wake import NOJ
 from py_wake.wind_turbines import OneTypeWindTurbines
-from py_wake.flow_map import HorizontalGrid
-from py_wake.tests.check_speed import timeit
 import matplotlib.pyplot as plt
+from py_wake.site.xrsite import XRSite
+import shutil
+
+
+@pytest.fixture(autouse=True)
+def close_plots():
+    try:
+        yield
+    finally:
+        try:
+            plt.close('all')
+        except Exception:
+            pass
 
 
 @pytest.fixture
@@ -29,10 +40,10 @@ def site2():
 
 
 def test_WaspGridSiteDistanceClass(site):
-    wgs = WaspGridSite(site._ds, distance=TerrainFollowingDistance(distance_resolution=2000))
+    wgs = XRSite(site.ds, distance=TerrainFollowingDistance(distance_resolution=2000))
     assert wgs.distance.distance_resolution == 2000
     assert wgs.distance.__call__.__func__ == TerrainFollowingDistance().__call__.__func__
-    wgs = WaspGridSite(site._ds, distance=StraightDistance())
+    wgs = XRSite(site.ds, distance=StraightDistance())
     assert wgs.distance.__call__.__func__ == StraightDistance().__call__.__func__
 
 
@@ -53,7 +64,7 @@ def test_local_wind(site):
 
 
 def test_shear(site):
-    npt.assert_array_almost_equal(site._ds['spd'].sel(x=262878, y=6504714, sec=1), [.6240589, .8932919])
+    npt.assert_array_almost_equal(site.ds['Speedup'].sel(x=262878, y=6504714, wd=0), [.6240589, .8932919])
     x = [262878.0001] * 3
     y = [6504714.0001] * 3
     z = [30, 115, 200]
@@ -62,6 +73,7 @@ def test_shear(site):
     if 0:
         plt.plot(ws, z, '.-')
         plt.show()
+    plt.close('all')
 
     # linear interpolation
     npt.assert_array_almost_equal(ws, [6.240589, np.mean([6.240589, 8.932919]), 8.932919])
@@ -79,7 +91,7 @@ def test_wasp_resources_grid_point(site):
     # 0.7221162    4.606324    17.96417    11.45838
     # 0.8088576    8.196074    16.16308    9.277925
     # 0.8800673    3.932325    14.82337    5.380589
-    # 0.8726974    -3.199536    19.99724    -1.433086""".split("\n")], dtype=np.float)
+    # 0.8726974    -3.199536    19.99724    -1.433086""".split("\n")], dtype=float)
     #     for x_ in x.T:
     #         print(list(x_))
     x = [262978]
@@ -123,9 +135,10 @@ def test_wasp_resources_grid_point(site):
     wt = OneTypeWindTurbines.from_tabular(name="NEG-Micon 2750/92 (2750 kW)", diameter=92, hub_height=70,
                                           ws=wt_u, ct=wt_ct, power=wt_p, power_unit='kw')
 
-    A_lst, k_lst, f_lst, spd_lst, orog_trn_lst, flow_inc_lst, tke_lst = [site.interp_funcs[n](
-        (x, y, 30, range(0, 360, 30))) for n in ['A', 'k', 'f', 'spd', 'orog_trn', 'flow_inc', 'tke']]
-    f_lst = f_lst * 360 / 12
+    lw = site.local_wind(x, y, 30, wd=range(0, 360, 30))
+    A_lst, k_lst, f_lst, spd_lst, orog_trn_lst, flow_inc_lst, ti15ms_lst = [
+        site.interp(site.ds[n], lw.coords).sel(i=0).values
+        for n in ['Weibull_A', 'Weibull_k', 'Sector_frequency', 'Speedup', 'Turning', 'flow_inc', 'ti15ms']]
     pdf_lst = [lambda x, A=A, k=k: k / A * (x / A)**(k - 1) * np.exp(-(x / A)**k) * (x[1] - x[0])
                for A, k in zip(A_lst, k_lst)]
 #     cdf_lst = [lambda x, A=A, k=k: 1 - np.exp(-(x / A) ** k) for A, k in zip(A_lst, k_lst)]
@@ -139,7 +152,7 @@ def test_wasp_resources_grid_point(site):
     npt.assert_array_almost_equal(spd_lst, wasp_spd)
     npt.assert_array_almost_equal(orog_trn_lst, wasp_trn)
     npt.assert_array_almost_equal(flow_inc_lst, wasp_inc)
-    npt.assert_array_almost_equal(tke_lst, np.array(wasp_ti) / 100)
+    npt.assert_array_almost_equal(ti15ms_lst, np.array(wasp_ti) / 100)
 
     # compare pdf, u_mean and aep to wasp
     lw = site.local_wind(x, np.array(y) + 1e-6, 30, wd=np.arange(0, 360, 30), ws=ws)
@@ -183,9 +196,9 @@ def test_wasp_resources_grid_point(site):
      [-0, 207, 477, 710, 1016, 1236, 1456, 1799])])
 def test_distances(site, dw_ref):
     x, y = site.initial_position.T
-    dw_ijl, cw_ijl, dh_ijl, _ = site.distances(src_x_i=x, src_y_i=y, src_h_i=np.array([70]),
-                                               dst_x_j=x, dst_y_j=y, dst_h_j=np.array([70]),
-                                               wd_il=np.array([[0]]))
+    site.distance.setup(src_x_i=x, src_y_i=y, src_h_i=np.array([70]),
+                        dst_xyh_j=(x, y, np.array([70])))
+    dw_ijl, cw_ijl, dh_ijl = site.distance(WD_il=np.array([[0]]))
     npt.assert_almost_equal(dw_ijl[0, :, 0], dw_ref)
 
     cw_ref = [236.1, 0., -131.1, -167.8, -204.5, -131.1, -131.1, -45.4]
@@ -196,9 +209,9 @@ def test_distances(site, dw_ref):
 def test_distances_different_points(site2):
     site, x, y = site2
     with pytest.raises(NotImplementedError):
-        site.distances(src_x_i=x, src_y_i=y, src_h_i=np.array([70]),
-                       dst_x_j=x[1:], dst_y_j=y[1:], dst_h_j=np.array([70]),
-                       wd_il=np.array([[0]]))
+        site.distance.setup(src_x_i=x, src_y_i=y, src_h_i=np.array([70]),
+                            dst_xyh_j=(x[1:], y[1:], np.array([70])))
+        site.distance(WD_il=np.array([[0]]))
 
 
 # def test_distances_wd_shape():
@@ -227,9 +240,12 @@ def test_speed_up_using_pickle():
     npt.assert_array_less(time_w_pkl * 10, time_wo_pkl)
 
 
-def test_interp_funcs_initialization_missing_key(site):
-    site = ParqueFicticioSite(distance=TerrainFollowingDistance2())
-    site.interp_funcs_initialization(['missing'])
+def test_speed_up_using_pickle_wrong_pkl():
+    pkl_fn = ParqueFicticio_path + "ParqueFicticio.pkl"
+    if os.path.exists(pkl_fn):
+        os.remove(pkl_fn)
+    shutil.copy(__file__, pkl_fn)
+    site = WaspGridSite.from_wasp_grd(ParqueFicticio_path, speedup_using_pickle=True)
 
 
 def test_one_layer():
@@ -252,25 +268,17 @@ def test_elevation(site):
 
 
 def test_plot_map(site):
-    import matplotlib.pyplot as plt
-    with pytest.raises(AttributeError, match="missing not found in dataset. Available data variables are:\nflow_inc,"):
-        site.plot_map('missing')
-
-    with pytest.raises(AttributeError, match=r"Sector None not found. Available sectors are: \[ 1"):
-        site.plot_map('ws_mean')
-    with pytest.raises(AttributeError, match="Height missing for 'ws_mean'"):
-        site.plot_map('ws_mean', sector=1)
-
-    site.plot_map('elev')
+    site.ds.Elevation.plot()
     plt.figure()
-    site.plot_map('ws_mean', 80, sector=1)
+    site.ds.ws_mean.sel(h=200, wd=0).plot()
     if 0:
         plt.show()
-    plt.close()
+    plt.close('all')
 
 
 def test_elevation_outside_map(site):
-    site.plot_map('elev')
+
+    site.ds.Elevation.plot()
     x = np.linspace(262500, 265500, 500)
     y = x * 0 + 6505450
     plt.plot(x, y, '--', label='Terrain line')
@@ -280,6 +288,7 @@ def test_elevation_outside_map(site):
     if 0:
         plt.legend()
         plt.show()
+    plt.close('all')
 
 
 def test_plot_ws_distribution(site):
@@ -295,17 +304,19 @@ def test_plot_ws_distribution(site):
                                                  0.0013, 0.0003, 0.0, 0.0, 0.0], 4)
     npt.assert_array_almost_equal(p2[-1, ::30], [0.0001, 0.0036, 0.0047, 0.0033, 0.0014,
                                                  0.0004, 0.0001, 0.0, 0.0, 0.0], 4)
+    plt.close('all')
 
 
 def test_plot_wd_distribution(site):
     x, y = site.initial_position[0]
     p = site.plot_wd_distribution(x=x, y=y, h=70, n_wd=12, ax=plt)
     # print(np.round(p, 3).tolist())
-    npt.assert_array_almost_equal(p, [0.052, 0.043, 0.058, 0.085, 0.089, 0.061,
-                                      0.047, 0.083, 0.153, 0.152, 0.108, 0.068], 3)
 
     if 0:
         plt.show()
+    plt.close('all')
+    npt.assert_array_almost_equal(p, [0.052, 0.043, 0.058, 0.085, 0.089, 0.061,
+                                      0.047, 0.083, 0.153, 0.152, 0.108, 0.068], 3)
 
 
 def test_plot_wd_distribution_with_ws_levels(site):
@@ -326,15 +337,17 @@ def test_plot_wd_distribution_with_ws_levels(site):
                                       [0.048, 0.074, 0.026, 0.003, 0.0],
                                       [0.044, 0.046, 0.015, 0.002, 0.0],
                                       [0.041, 0.023, 0.003, 0.0, 0.0]], 3)
+    plt.close('all')
 
 
 def test_additional_input():
     site = ParqueFicticioSite()
-    wgs = WaspGridSite(site._ds, distance=TerrainFollowingDistance(distance_resolution=2000))
-    wgs.interp_funcs_initialization(['ws_mean'])
+
     x, y = site.initial_position.T
     h = 70 * np.ones_like(x)
-    ws_mean, = wgs.interpolate(['ws_mean'], x, y, h)
+
+    lw = site.local_wind(x, y, h)
+    ws_mean = site.interp(site.ds.ws_mean, lw.coords)
     npt.assert_array_almost_equal(ws_mean[0, :50],
                                   np.array([4.77080802, 4.77216214, 4.77351626, 4.77487037, 4.77622449,
                                             4.77757861, 4.77893273, 4.78028685, 4.78164097, 4.78299508,
@@ -348,36 +361,5 @@ def test_additional_input():
                                             5.70707884, 5.76678866, 5.82649848, 5.8862083, 5.94591812]))
 
 
-def test_interpolation_speed():
-    import xarray as xr
-    da = xr.DataArray(np.sin(0.3 * np.arange(20).reshape(5, 4)),
-                      [('x', np.arange(5)),
-                       ('y', [0.1, 0.2, 0.3, 0.4])])
-    x = xr.DataArray([0.5, 1.5, 2.5], dims='z')
-    y = xr.DataArray([0.15, 0.25, 0.35], dims='z')
-    da.interp(x=x, y=y)
-
-    site = ParqueFicticioSite()
-    x, y = site.initial_position.T
-    X, Y, x_j, y_j, h_j = HorizontalGrid()(x, y, 70)
-    wd = [270]  # site.default_wd
-    ws = site.default_ws
-    res1, t_lst = timeit(site.interp_funcs['A'])((x_j, y_j, h_j, x_j * 0 + 270))
-    print(res1.shape)
-    res2, t_lst = timeit(lambda x, y, z, sec:
-                         site._ds.A.interp(x=xr.DataArray(x, dims='z'),
-                                           y=xr.DataArray(y, dims='z'),
-                                           z=xr.DataArray(z, dims='z'),
-                                           sec=xr.DataArray(sec, dims='z')).data)(x_j, y_j, h_j, x_j * 0 + 10)
-    npt.assert_array_almost_equal(res1, res2)
-    if 0:
-        c = plt.contourf(X, Y, res1.reshape(X.shape))
-        plt.colorbar(c)
-        plt.figure()
-        c = plt.contourf(X, Y, res2.reshape(X.shape))
-        plt.colorbar(c)
-        plt.show()
-
-
 if __name__ == '__main__':
     test_wasp_resources_grid_point(ParqueFicticioSite())
